/* 
richards.c
calculation of soil water content layer by layer taking into account soil hydrological processes 
(percolation, diffusion)  using inamic discretization level in order to taking account the change of the diffusivity and conductivity depending on soil moisture 
(after rain event the soil moisture rises suddenly causing high diffusivity and conductivity. If we assumed that these diffusivity 
 and conductivity values were valid all the day we would overestimate the velocity diffusion and percolation process) 

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
Biome-BGCMuSo v6.2.
Copyright 2020, D. Hidy [dori.hidy@gmail.com]
Hungarian Academy of Sciences, Hungary
See the website of Biome-BGCMuSo at http://nimbus.elte.hu/bbgc/ for documentation, model executable and example input files.
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "ini.h"
#include "bgc_struct.h"
#include "bgc_constants.h"
#include "bgc_func.h"
#define MIN(X, Y) (((X) < (Y)) ? (X) : (Y))
#define MAX(X, Y) (((X) > (Y)) ? (X) : (Y))   

int richards(siteconst_struct* sitec, soilprop_struct* sprop, const epconst_struct* epc, epvar_struct* epv, wstate_struct* ws, wflux_struct* wf)
{

	
	/* internal variables */
	int errorCode, fr;
	double INFILT,INFILT_act,INFILT_sum;


	/* diffusion and percolation calculation */
	double D0, D1;	        /* hydrological diffusion coefficient (m2/s) */

	double diffus, percol, VWC_diff_max;
	double localvalue,  exponent_discretlevel;
	double soilw0, soilw1, VWC0, VWC1, VWC_diff0, VWC_diff1, soilw_sat0, soilw_fc0;
	int n_second, step, n_sec, discretlevel;

	double dz0, dz1;
	int inORout;

	richards_struct rich;

	errorCode = 0;

	INFILT_sum=0;

	/* parameter values */
	rich.critDepth = 1;
	rich.dz_aboveCD = 0.01;
	rich.dz_belowCD = 0.1;

	/* --------------------------------------------------------------------------------------------------------*/
	/* 1. INITALIZATION */
	
	/* initialization */
	
	n_second=step=0;
	n_sec=1;
	discretlevel = epc->discretlevel_Richards + 3; /* discretization level: change in VWC 1% -> timestep: second */

	/* change 10 layers to 10000 layers */
	inORout=0;
	if (richards_fineRES(inORout, sitec, sprop, epv, ws, wf, &rich))
	{
		if (!errorCode) 
		{
			printf("\n");
			printf("ERROR: richards_fineRES() for richards \n");
		}
		errorCode=1;
	}



	INFILT = wf->infiltPOT;
	INFILT_act = INFILT/NSEC_IN_DAY;
	
	
	/* --------------------------------------------------------------------------------------------------------*/	
	/* 3. CALCULATE PROCESSES WITH DYNAMIC TIME STEP (except of bottom layer) */

	while (n_second < NSEC_IN_DAY)
	{
	
		VWC_diff_max=0;

		/* ********************************/	
		/* 4.1 CALCULATE PROCESSES LAYER TO LAYER (except of bottom layer) */
		for (fr=0 ; fr < fineRES-1; fr++)
		{
		
			
			/* 4.1.1 INITALIZATION */
			dz0        = rich.dz_FR[fr];
			soilw0     = rich.soilw_FR[fr];
			VWC0       = rich.VWC_FR[fr];
			soilw_sat0 = rich.VWCsat_FR[fr] * dz0 * water_density;
			soilw_fc0  = rich.VWCfc_FR[fr] * dz0 * water_density;
		
			dz1        = rich.dz_FR[fr+1];
			soilw1     = rich.soilw_FR[fr+1];
			VWC1       = rich.VWC_FR[fr+1];
		

			/* -----------------------------*/
			/* 4.1.1. INFILTRATION */
  			if (fr == 0) 
			{
				soilw0 += INFILT_act;
					
				if ((soilw0 - soilw_sat0) > CRIT_PREC)			
				{
					wf->soilw_to_pondw += (soilw0 - soilw_sat0);
					INFILT_act         -= (soilw0 - soilw_sat0);
					INFILT             -= (soilw0 - soilw_sat0);
					soilw0				= soilw_sat0;
					VWC0                = rich.VWCsat_FR[fr];

				}
				else
					VWC0 = soilw0 / (dz0 * water_density);

				INFILT           -= INFILT_act;
				INFILT_sum       += INFILT_act;

		
			}
			/* -----------------------------*/
			/* 4.1.2. PERCOLATION */

			/* conductivity coefficient - theoretical upper limit: saturation value */
			rich.Kact_FR[fr] = rich.Ksat_FR[fr] * pow(VWC0/rich.VWCsat_FR[fr], 2*(rich.soilB_FR[fr])+3);


			/* percolation flux - only if VWC is above field capavity*/
			if ((VWC0 - rich.VWCfc_FR[fr]) > CRIT_PREC)
			{
				percol = rich.Kact_FR[fr] * water_density * n_sec; // kg/m2

				/* limitation of percolation flux -> field capacity of given layer */
				if ((soilw0 - percol) - soilw_fc0 < 0) 
				{
					percol -= (soilw_fc0 - (soilw0 - percol)); 
				}
			}
			else
				percol = 0;
	
		
			soilw0 = soilw0 - percol;
			VWC0   = soilw0 / dz0 / water_density;
			soilw1 = soilw1 + percol;
			VWC1   = soilw1 / dz1 / water_density;

			/* -----------------------------*/
			/* 4.1.3. DIFFUSION */
		
			/* diffusivity coefficient 	*/
	       	D0 = (((rich.soilB_FR[fr]   * rich.Ksat_FR[fr]   * (-100*rich.PSIsat_FR[fr])))   / VWC0) * pow(VWC0/rich.VWCsat_FR[fr],   rich.soilB_FR[fr]+2);
			
			D1 = (((rich.soilB_FR[fr+1] * rich.Ksat_FR[fr+1] * (-100*rich.PSIsat_FR[fr+1]))) / VWC1) * pow(VWC1/rich.VWCsat_FR[fr+1], rich.soilB_FR[fr+1]+2);

			rich.Dact_FR[fr]  = (D0 + D1) /2;
	
		
			/* diffusion flux  */
			diffus = (VWC0 - VWC1)/((dz0+dz1)/2) 	* rich.Dact_FR[fr]  * water_density;

			soilw0 = soilw0 - diffus;
			VWC0   = soilw0 / dz0 / water_density;
			soilw1 = soilw1 + diffus;
			VWC1   = soilw1 / dz1 / water_density;

			/* -----------------------------*/
			/* 4.1.5. FINDING THE MAXIMAL CHANGE to determine discretization step */
			
			VWC_diff0 = fabs(rich.VWC_FR[fr]  - VWC0);
			VWC_diff1 = fabs(rich.VWC_FR[fr+1] - VWC1);
		
			if (VWC_diff_max < VWC_diff0 || VWC_diff_max < VWC_diff1) VWC_diff_max = MAX(VWC_diff0, VWC_diff1);		
		
		
		
			/* -----------------------------*/
			/* 4.1.6. UPDATING STATE VARIABLES    */

			rich.percol_FR[fr]   += percol;
			rich.diffus_FR[fr]   += diffus;

			rich.soilw_FR[fr]    = soilw0;
			rich.VWC_FR[fr]      = VWC0;
			
			rich.soilw_FR[fr+1]  = soilw1;
			rich.VWC_FR[fr+1]    = VWC1;

			
		}
		n_second += n_sec;

		/* ********************************/	
		/* 4.2. CALCULATION OF DISCRETE LEVEL: if the maximal change of VWC is not equal to 0 (greater than) -> discretlevel is the function of local value of the change */
		VWC_diff_max = VWC_diff_max/n_sec;
		if (VWC_diff_max > CRIT_PREC)
		{
			localvalue=floor(log10(VWC_diff_max));
			if (localvalue + discretlevel < 0)
				exponent_discretlevel = fabs(localvalue + discretlevel);
			else
				exponent_discretlevel = 0;

			n_sec = (int) pow(10, exponent_discretlevel);
			if (n_sec + n_second > NSEC_IN_DAY) 
				n_sec = NSEC_IN_DAY - n_second;
		}
		else
		{
			n_sec = NSEC_IN_DAY - n_second;
		}
		VWC_diff_max = 0;
		if (n_second < NSEC_IN_DAY)
			INFILT_act = INFILT/(NSEC_IN_DAY-n_second) * n_sec;
		else
			INFILT_act = 0;


		step=step+1;	
	}

	/* change 140 layers to 10 layers */
	inORout=1;
	if (richards_fineRES(inORout, sitec, sprop, epv, ws, wf, &rich))
	{
		if (!errorCode) 
		{
			printf("\n");
			printf("ERROR: richards_fineRES() for richards \n");
		}
		errorCode=1;
	}	

	return (errorCode);
}


int richards_fineRES(int inORout, siteconst_struct* sitec, soilprop_struct* sprop, epvar_struct* epv, 
	                 wstate_struct* ws, wflux_struct* wf, richards_struct* rich)
{	
	int nlayer, fr, errorCode, layerbound;
	double depth, dz;
	nlayer=errorCode=layerbound = 0;


	/* new VWC values after Richards-method */
	if (inORout == 1)
	{
		for (nlayer = 0; nlayer < N_SOILLAYERS-1; nlayer++) ws->soilw[nlayer] = 0;
		nlayer=0;
	}

	/*  depth of the first layer in fine resolution */
	depth=rich->dz_aboveCD;  


	for (fr=0 ; fr < fineRES-1; fr++)
	{
		if (depth - rich->critDepth > CRIT_PREC)
			dz=rich->dz_belowCD;
		else
			dz=rich->dz_aboveCD;

		if (fabs(depth - sitec->soillayer_depth[nlayer]) < CRIT_PREC)
			layerbound=1;
		else
			layerbound=0;
	
		/* input for Richards */
		if (inORout == 0)
		{
			rich->VWC_FR[fr]      = epv->VWC[nlayer];
			rich->VWCfc_FR[fr]    = sprop->VWCfc[nlayer];
			rich->VWCsat_FR[fr]   = sprop->VWCsat[nlayer];
			rich->PSIsat_FR[fr]   = sprop->PSIsat[nlayer];
			rich->soilB_FR[fr]    = sprop->soilB[nlayer];
			rich->Ksat_FR[fr]     = sprop->hydrCONDUCTsat[nlayer];
			rich->Dsat_FR[fr]     = sprop->hydrDIFFUSsat[nlayer];
			rich->Kact_FR[fr]     = epv->hydrCONDUCTact[nlayer];
			rich->Dact_FR[fr]     = epv->hydrDIFFUSact[nlayer];
			rich->percol_FR[fr]   = 0;
			rich->diffus_FR[fr]   = 0;
			rich->soilw_FR[fr]    = rich->VWC_FR[fr]*dz*water_density;
			rich->dz_FR[fr]       = dz;
		}

		/* output from Richards */
		else
		{
			/* averaged VWC */
			ws->soilw[nlayer]     += rich->soilw_FR[fr];

			/* percolation and diffusion from boundary fineRES layer */
			if (layerbound)
			{
				wf->soilw_percolated[nlayer] = rich->percol_FR[fr];
				wf->soilw_diffused[nlayer] = rich->diffus_FR[fr];
				epv->hydrCONDUCTact[nlayer] = rich->Kact_FR[fr];
				epv->hydrDIFFUSact[nlayer] = rich->Dact_FR[fr];
			}
		}

	
		if (layerbound) nlayer=nlayer+1;	

		if (fabs(depth - rich->critDepth) < CRIT_PREC)
			depth+=rich->dz_belowCD;
		else
			depth+=dz;
		
	}
	
	/* input for Richards:  bottom layer is special -> no VWC change */
	if (inORout == 0)
	{
		dz                           = sitec->soillayer_thickness[N_SOILLAYERS-1];
		rich->VWC_FR[fineRES-1]      = epv->VWC[N_SOILLAYERS-1];
		rich->VWCfc_FR[fineRES-1]    = sprop->VWCfc[N_SOILLAYERS-1];
		rich->VWCsat_FR[fineRES-1]   = sprop->VWCsat[N_SOILLAYERS-1];
		rich->PSIsat_FR[fineRES-1]   = sprop->PSIsat[N_SOILLAYERS-1];
		rich->soilB_FR[fineRES-1]    = sprop->soilB[N_SOILLAYERS-1];
		rich->Ksat_FR[fineRES-1]     = sprop->hydrCONDUCTsat[N_SOILLAYERS-1];
		rich->Dsat_FR[fineRES-1]     = sprop->hydrDIFFUSsat[N_SOILLAYERS-1];
		rich->Kact_FR[fineRES-1]     = epv->hydrCONDUCTact[fineRES-1];
		rich->Dact_FR[fineRES-1]     = epv->hydrDIFFUSact[fineRES-1];
		rich->percol_FR[fineRES-1]   = 0;
		rich->diffus_FR[fineRES-1]   = 0;
		rich->soilw_FR[fineRES-1]    = rich->VWC_FR[fineRES-1]*dz*water_density;
		rich->dz_FR[fineRES-1]       = dz;
	}

	/* calculation of VWC from soilw */
	if (inORout == 1)
	{
		for (nlayer = 0; nlayer < N_SOILLAYERS-1; nlayer++)  epv->VWC[nlayer] = ws->soilw[nlayer] / (water_density * sitec->soillayer_thickness[nlayer]);
		/* bottom layer is special -> no VWC change */
		wf->soilw_percolated[N_SOILLAYERS-1] = wf->soilw_percolated[N_SOILLAYERS-2];
		wf->soilw_diffused[N_SOILLAYERS-1]   = wf->soilw_diffused[N_SOILLAYERS-2];
	}



	


	return (errorCode);
}